"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const tool_lib_1 = require("@kettil/tool-lib");
const executableError_1 = __importDefault(require("../errors/executableError"));
const exec = ({ cmd, args = [], output, cwd = process.cwd(), env = tool_lib_1.env(), log, }) => new Promise((resolve, reject) => {
    const command = `${cmd} ${args.join(' ')}`;
    log.debug(`\nexec: ${command}`);
    const stream = child_process_1.spawn(cmd, args, { cwd, env, shell: true });
    let stdout = '';
    let stderr = '';
    stream.stdout.on('data', (data) => {
        stdout += String(data);
        log.debug(String(data));
        if (output) {
            output(String(data));
        }
    });
    stream.stderr.on('data', (data) => {
        stderr += String(data);
        log.debug(String(data));
        if (output) {
            output(String(data));
        }
    });
    stream.on('close', (code) => {
        if (code !== 0) {
            reject(new executableError_1.default(stderr, command, code, stdout, stderr));
        }
        else {
            resolve({ stdout, stderr });
        }
    });
});
exports.default = exec;
