"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.desc = exports.command = void 0;
const packageLoad_1 = __importDefault(require("../actions/npm/packageLoad"));
const babel_1 = __importDefault(require("../actions/tools/babel"));
const tsc_1 = __importDefault(require("../actions/tools/tsc"));
const webpack_1 = __importDefault(require("../actions/tools/webpack"));
const spinner_1 = require("../lib/cli/spinner");
const yargs_1 = require("../lib/cli/yargs");
const logo_1 = __importDefault(require("../lib/logo"));
exports.command = 'build';
exports.desc = 'Run the code quality tools';
const options = { group: `${exports.command}-Options` };
exports.builder = yargs_1.builderDefault(exports.command, (yargs) => yargs.options({
    ci: { ...options, type: 'boolean', default: false, describe: 'Run it in CI mode' },
}));
const handler = async (argv) => {
    try {
        const isPrivate = await packageLoad_1.default(argv, { key: 'private' });
        if (argv.ci) {
            if (isPrivate !== true) {
                await tsc_1.default(argv, { mode: 'type-create' });
            }
            await babel_1.default(argv);
            await webpack_1.default(argv);
        }
        else {
            await logo_1.default();
            if (isPrivate !== true) {
                await spinner_1.spinnerAction(tsc_1.default(argv, { mode: 'type-create' }), 'Typescript');
            }
            await spinner_1.spinnerAction(babel_1.default(argv), 'Babel');
            await spinner_1.spinnerAction(webpack_1.default(argv), 'Webpack');
        }
    }
    catch (error) {
        yargs_1.errorHandler(argv, error);
    }
};
exports.handler = handler;
