"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.desc = exports.command = void 0;
const tool_lib_1 = require("@kettil/tool-lib");
const add_1 = __importDefault(require("../actions/git/add"));
const commit_1 = __importDefault(require("../actions/git/commit"));
const init_1 = __importDefault(require("../actions/git/init"));
const file_1 = __importDefault(require("../actions/node/file"));
const folder_1 = __importDefault(require("../actions/node/folder"));
const template_1 = __importDefault(require("../actions/node/template"));
const init_2 = __importDefault(require("../actions/npm/init"));
const install_1 = __importDefault(require("../actions/npm/install"));
const packageLoad_1 = __importDefault(require("../actions/npm/packageLoad"));
const packageUpdate_1 = __importDefault(require("../actions/npm/packageUpdate"));
const spinner_1 = require("../lib/cli/spinner");
const yargs_1 = require("../lib/cli/yargs");
const logo_1 = __importDefault(require("../lib/logo"));
exports.command = 'init';
exports.desc = 'Initializes the project';
const options = { type: 'boolean', default: false, group: `${exports.command}-Options:` };
exports.builder = yargs_1.builderDefault(exports.command, (yargs) => yargs.options({
    package: { ...options, alias: 'p', desc: 'Project is created as a package' },
    cli: { ...options, alias: 'c', implies: 'package', desc: 'Extends the package with CLI features' },
    github: { type: 'string', implies: 'package', desc: 'Github username', group: `${exports.command}-Options:` },
    // conflict with cli
    // react: { ...options, alias: 'r', default: undefined, desc: 'React will be installed', },
    noCommit: { ...options, desc: 'No initial commit is executed at the end' },
}));
const handler = async (argv) => {
    var _a;
    try {
        await logo_1.default();
        const githubUsername = (_a = argv.github) === null || _a === void 0 ? void 0 : _a.trim();
        if (githubUsername !== undefined && githubUsername === '') {
            throw new Error('The github name is empty');
        }
        const files = [];
        const folders = ['.vscode', 'src', 'src/lib', 'tests'];
        const templates = [
            ['vscode/settings.json', '.vscode/settings.json'],
            ['editorconfig', '.editorconfig'],
            ['huskyrc.json', '.huskyrc'],
            ['eslintignore', '.eslintignore'],
            ['gitignore', '.gitignore'],
            ['npmignore', '.npmignore'],
            ['prettierignore', '.prettierignore'],
        ];
        const libraryProduction = [];
        const libraryDevelopment = ['husky', 'carna'];
        const packageBin = {};
        const packageInit = {
            private: true,
            version: '0.1.0',
            engines: { node: '>= 14', npm: '>=6' },
        };
        const packageUpdate = {};
        const packageScripts = {
            lint: 'npx carna lint',
        };
        const packagePeerDependencies = [];
        packageInit.author = 'name <email>';
        // ######################
        // # Github             #
        // ######################
        if (githubUsername) {
            folders.push('.dependabot', '.github/workflows');
            templates.push(['dependabot/config.yml', '.dependabot/config.yml']);
            templates.push(['github/CODEOWNERS', '.github/CODEOWNERS']);
            templates.push(['github/workflows/qa.yml', '.github/workflows/qa.yml']);
        }
        // ######################
        // # Library            #
        // ######################
        if (argv.package) {
            if (githubUsername) {
                packageInit.repository = { type: 'git', url: `https://github.com/${githubUsername}/<repo>` };
                packageInit.bugs = { url: `https://github.com/${githubUsername}/<repo>/issues/new` };
                templates.push(['github/workflows/release.yml', '.github/workflows/release.yml']);
            }
            // @todo Temporary until command release
            templates.push(['releaserc.json', '.releaserc.json']);
            // libraryDevelopment.push('@kettil/semantic-release-config');
            packageInit.publishConfig = { access: 'public' };
        }
        // ######################
        // # CLI                #
        // ######################
        if (argv.cli) {
            folders.push('src/bin');
            templates.push(['src/bin/index.ts']);
        }
        // ######################
        // # Babel/Webpack      #
        // ######################
        templates.push(['babel.config.js']);
        templates.push(['webpack.config.js']);
        libraryDevelopment.push('@babel/cli', '@babel/core', '@babel/plugin-transform-runtime', '@babel/preset-env', '@babel/preset-typescript', 'babel-loader', 'webpack', 'webpack-cli');
        if (argv.package && !argv.cli) {
            libraryDevelopment.push('@babel/runtime-corejs3');
            packagePeerDependencies.push('@babel/runtime-corejs3');
        }
        else {
            libraryProduction.push('@babel/runtime-corejs3');
        }
        // ######################
        // # Typescript         #
        // ######################
        files.push('src/index.ts', 'src/lib/types.ts');
        templates.push(['typescriptrc.json', 'tsconfig.json']);
        libraryDevelopment.push('@types/node', 'typescript');
        packageUpdate.module = 'build/index.js';
        packageUpdate.types = 'build/index.d.ts';
        // ######################
        // # Build              #
        // ######################
        packageScripts.build = 'npx carna build';
        packageScripts.prebuild = 'rm -rf ./build';
        // ######################
        // # React              #
        // ######################
        /*
      // PROD or DEV (with peerDependencies)
      "react"
      "react-dom"
    
      if (argv.package) {
        packagePeerDependencies.push('@babel/preset-react', '@types/react', '@types/react-dom');
      }
      */
        // ######################
        // # Jest               #
        // ######################
        templates.push(['jest.config.js'], ['jest.ci.js']);
        libraryDevelopment.push('jest', '@types/jest', 'babel-jest');
        files.push('src/index.test.ts');
        packageScripts.test = 'jest --selectProjects unit';
        packageScripts['test:integration'] = 'jest --selectProjects integration';
        packageScripts['test:watch'] = 'npm run test -- --watch';
        packageScripts['test:integration:watch'] = 'npm run test:integration -- --watch';
        // ######################
        // # Actions            #
        // ######################
        // NPM
        await spinner_1.spinnerAction(init_2.default(argv, { settings: { ...packageInit } }), 'Create the package.json');
        const packageName = await packageLoad_1.default(argv, { key: 'name', throwError: true });
        if (typeof packageName !== 'string') {
            throw new TypeError('Package name could not be read');
        }
        packageUpdate.main = `build/${packageName}.js`;
        if (argv.cli) {
            packageBin[packageName] = 'build/bin/index.js';
        }
        // GIT
        await spinner_1.spinnerAction(init_1.default(argv), 'Create the git repository');
        // FOLDER
        await spinner_1.spinnerAction(Promise.all(folders.map((folder) => folder_1.default(argv, { folder }))), 'Create the project folders');
        // FILES
        await spinner_1.spinnerAction(Promise.all(files.map((file) => file_1.default(argv, { file }))), 'Create the project files');
        // TEMPLATES
        /* eslint-disable @typescript-eslint/naming-convention */
        const variables = {
            GITHUB_USERNAME: githubUsername,
            PACKAGE_LIBRARY: tool_lib_1.toCamelCase(packageName),
            PACKAGE_FILENAME: packageName,
        };
        if (!argv.package || argv.cli) {
            variables.BABEL_MODULE_TYPE = 'commonjs';
            variables.BABEL_MODULE_TARGET = 'defaults';
        }
        else {
            variables.BABEL_MODULE_TYPE = false;
            variables.BABEL_MODULE_TARGET = { node: '14' };
        }
        /* eslint-enable @typescript-eslint/naming-convention */
        await spinner_1.spinnerAction(Promise.all(templates.map(([source, target]) => template_1.default(argv, { source, target, variables }))), 'Create template files');
        // NPM INSTALL PROD
        await spinner_1.spinnerAction(install_1.default(argv, { packages: libraryProduction, mode: 'prod' }), 'Install prod dependencies');
        // NPM INSTALL DEV
        await spinner_1.spinnerAction(install_1.default(argv, { packages: libraryDevelopment, mode: 'dev' }), 'Install dev dependencies');
        // UPDATE PACKAGE.JSON
        await spinner_1.spinnerAction(packageUpdate_1.default(argv, {
            settings: {
                ...packageUpdate,
                bin: packageBin,
                scripts: packageScripts,
                peerDependencies: packagePeerDependencies,
            },
        }), 'Update the package.json');
        if (!argv.noCommit) {
            // GIT ADD
            await spinner_1.spinnerAction(add_1.default(argv, { files: ['.'] }), 'Add files to repository');
            // GIT COMMIT
            await spinner_1.spinnerAction(commit_1.default(argv, { msg: 'feat: 🐣' }), 'Create the initial commit');
        }
    }
    catch (error) {
        yargs_1.errorHandler(argv, error);
    }
};
exports.handler = handler;
